@@ -23,6 +23,7 @@ from idds.orm import (transforms as orm_transforms,
                       contents as orm_contents,
                       messages as orm_messages,
                       processings as orm_processings)
+from idds.core import messages as core_messages
 
 
 @transactional_session
@@ -110,7 +111,7 @@ def get_transform_ids(workprogress_id, request_id=None, workload_id=None, transf
 
 
 @read_session
-def get_transforms(workprogress_id=None, to_json=False, request_id=None, workload_id=None, transform_id=None, session=None):
+def get_transforms(request_id=None, workload_id=None, transform_id=None, to_json=False, session=None):
     """
     Get transforms or raise a NoObject exception.
 
@@ -122,13 +123,48 @@ def get_transforms(workprogress_id=None, to_json=False, request_id=None, workloa
 
     :returns: list of transform.
     """
-    return orm_transforms.get_transforms(workprogress_id=workprogress_id, request_id=request_id,
-                                         workload_id=workload_id, transform_id=transform_id,
+    return orm_transforms.get_transforms(request_id=request_id,
+                                         workload_id=workload_id,
+                                         transform_id=transform_id,
                                          to_json=to_json, session=session)
 
 
 @transactional_session
-def get_transforms_by_status(status, period=None, locking=False, bulk_size=None, to_json=False, by_substatus=False, session=None):
+def get_transforms_with_messaging(locking=False, bulk_size=None, session=None):
+    msgs = core_messages.retrieve_transform_messages(transform_id=None, bulk_size=bulk_size, session=session)
+    if msgs:
+        tf_ids = [msg['transform_id'] for msg in msgs]
+        if locking:
+            tf2s = orm_transforms.get_transforms_by_status(status=None, transform_ids=tf_ids,
+                                                           locking=locking, locking_for_update=True,
+                                                           bulk_size=None, session=session)
+            if tf2s:
+                transforms = []
+                for tf_id in tf_ids:
+                    if len(transforms) >= bulk_size:
+                        break
+                    for tf in tf2s:
+                        if tf['transform_id'] == tf_id:
+                            transforms.append(tf)
+                            break
+            else:
+                transforms = []
+
+            parameters = {'locking': TransformLocking.Locking}
+            for tf in transforms:
+                orm_transforms.update_transform(transform_id=tf['transform_id'], parameters=parameters, session=session)
+            return transforms
+        else:
+            transforms = orm_transforms.get_transforms_by_status(status=None, transform_ids=tf_ids, locking=locking,
+                                                                 locking_for_update=locking,
+                                                                 bulk_size=bulk_size, session=session)
+            return transforms
+    else:
+        return []
+
+
+@transactional_session
+def get_transforms_by_status(status, period=None, locking=False, bulk_size=None, to_json=False, by_substatus=False, with_messaging=False, session=None):
     """
     Get transforms or raise a NoObject exception.
 
@@ -141,6 +177,11 @@ def get_transforms_by_status(status, period=None, locking=False, bulk_size=None,
 
     :returns: list of transform.
     """
+    if with_messaging:
+        transforms = get_transforms_with_messaging(locking=locking, bulk_size=bulk_size, session=session)
+        if transforms:
+            return transforms
+
     if locking:
         if bulk_size:
             # order by cannot work together with locking. So first select 2 * bulk_size without locking with order by.
@@ -206,7 +247,7 @@ def update_transform(transform_id, parameters, session=None):
 def add_transform_outputs(transform, transform_parameters, input_collections=None, output_collections=None, log_collections=None,
                           update_input_collections=None, update_output_collections=None, update_log_collections=None,
                           new_contents=None, update_contents=None, new_processing=None, update_processing=None,
-                          messages=None, message_bulk_size=10000, session=None):
+                          messages=None, update_messages=None, message_bulk_size=10000, session=None):
     """
     For input contents, add corresponding output contents.
 
@@ -276,36 +317,22 @@ def add_transform_outputs(transform, transform_parameters, input_collections=Non
     if messages:
         if not type(messages) in [list, tuple]:
             messages = [messages]
-        for message in messages:
-            orm_messages.add_message(msg_type=message['msg_type'],
-                                     status=message['status'],
-                                     source=message['source'],
-                                     request_id=message['request_id'],
-                                     workload_id=message['workload_id'],
-                                     transform_id=message['transform_id'],
-                                     num_contents=message['num_contents'],
-                                     msg_content=message['msg_content'],
-                                     bulk_size=message_bulk_size,
-                                     session=session)
-
-    """
-    if to_cancel_processing:
-        to_cancel_params = {'status': ProcessingStatus.Cancel}
-        for to_cancel_id in to_cancel_processing:
-            orm_processings.update_processing(processing_id=to_cancel_id, parameters=to_cancel_params, session=session)
-    processing_id = None
-    if processing:
-        processing_id = orm_processings.add_processing(**processing, session=session)
-    """
+        # for message in messages:
+        #     orm_messages.add_message(msg_type=message['msg_type'],
+        #                              status=message['status'],
+        #                              source=message['source'],
+        #                              request_id=message['request_id'],
+        #                              workload_id=message['workload_id'],
+        #                              transform_id=message['transform_id'],
+        #                              num_contents=message['num_contents'],
+        #                              msg_content=message['msg_content'],
+        #                              bulk_size=message_bulk_size,
+        #                              session=session)
+        orm_messages.add_messages(messages, session=session)
+    if update_messages:
+        orm_messages.update_messages(update_messages, session=session)
 
     if transform:
-        """
-        if processing_id is not None:
-            if not transform['transform_metadata']:
-                transform['transform_metadata'] = {'processing_id': processing_id}
-            else:
-                transform['transform_metadata']['processing_id'] = processing_id
-        """
         if processing_id:
             # work.set_processing_id(new_processing, processing_id)
             work.set_processing_id(new_processing['processing_metadata']['processing'], processing_id)
@@ -401,29 +428,94 @@ def release_inputs(to_release_inputs):
     return update_contents
 
 
-def release_inputs_by_collection(to_release_inputs):
+def release_inputs_by_collection_old(to_release_inputs):
     update_contents = []
     for coll_id in to_release_inputs:
         to_release_contents = to_release_inputs[coll_id]
         if to_release_contents:
             to_release = to_release_contents[0]
-            to_release_names = []
+            to_release_names_available = []
+            to_release_names_fake_available = []
+            to_release_names_final_failed = []
+            to_release_names_missing = []
             for to_release_content in to_release_contents:
-                if (to_release_content['status'] in [ContentStatus.Available]                  # noqa: W503
-                    or to_release_content['substatus'] in [ContentStatus.Available]):          # noqa: W503
-                    to_release_names.append(to_release_content['name'])
+                if (to_release_content['status'] in [ContentStatus.Available]            # noqa: W503
+                    or to_release_content['substatus'] in [ContentStatus.Available]):    # noqa: W503
+                    to_release_names_available.append(to_release_content['name'])
+                elif (to_release_content['status'] in [ContentStatus.FakeAvailable]            # noqa: W503
+                      or to_release_content['substatus'] in [ContentStatus.FakeAvailable]):    # noqa: W503
+                    to_release_names_fake_available.append(to_release_content['name'])
+                elif (to_release_content['status'] in [ContentStatus.FinalFailed]            # noqa: W503
+                      or to_release_content['substatus'] in [ContentStatus.FinalFailed]):    # noqa: W503
+                    to_release_names_final_failed.append(to_release_content['name'])
+                elif (to_release_content['status'] in [ContentStatus.Missing]            # noqa: W503
+                      or to_release_content['substatus'] in [ContentStatus.Missing]):    # noqa: W503
+                    to_release_names_missing.append(to_release_content['name'])
             contents = orm_contents.get_input_contents(request_id=to_release['request_id'],
                                                        coll_id=to_release['coll_id'],
                                                        name=None)
 
             for content in contents:
-                if (content['content_relation_type'] == ContentRelationType.InputDependency    # noqa: W503
-                    and content['status'] not in [ContentStatus.Available]                     # noqa: W503
-                    and content['name'] in to_release_names):                                  # noqa: W503
-                    update_content = {'content_id': content['content_id'],
-                                      'substatus': ContentStatus.Available,
-                                      'status': ContentStatus.Available}
-                    update_contents.append(update_content)
+                if (content['content_relation_type'] == ContentRelationType.InputDependency):    # noqa: W503
+                    if (content['status'] not in [ContentStatus.Available]                       # noqa: W503
+                        and content['name'] in to_release_names_available):                          # noqa: W503
+                        update_content = {'content_id': content['content_id'],
+                                          'substatus': ContentStatus.Available,
+                                          'status': ContentStatus.Available}
+                        update_contents.append(update_content)
+                    elif (content['status'] not in [ContentStatus.FakeAvailable]                     # noqa: W503
+                          and content['name'] in to_release_names_fake_available):                        # noqa: W503
+                        update_content = {'content_id': content['content_id'],
+                                          'substatus': ContentStatus.FakeAvailable,
+                                          'status': ContentStatus.FakeAvailable}
+                        update_contents.append(update_content)
+                    elif (content['status'] not in [ContentStatus.FinalFailed]                     # noqa: W503
+                          and content['name'] in to_release_names_final_failed):                        # noqa: W503
+                        update_content = {'content_id': content['content_id'],
+                                          'substatus': ContentStatus.FinalFailed,
+                                          'status': ContentStatus.FinalFailed}
+                        update_contents.append(update_content)
+                    elif (content['status'] not in [ContentStatus.Missing]                     # noqa: W503
+                          and content['name'] in to_release_names_missing):                        # noqa: W503
+                        update_content = {'content_id': content['content_id'],
+                                          'substatus': ContentStatus.Missing,
+                                          'status': ContentStatus.Missing}
+                        update_contents.append(update_content)
+    return update_contents
+
+
+def release_inputs_by_collection(to_release_inputs):
+    update_contents = []
+    status_to_check = [ContentStatus.Available, ContentStatus.FakeAvailable, ContentStatus.FinalFailed, ContentStatus.Missing]
+    for coll_id in to_release_inputs:
+        to_release_contents = to_release_inputs[coll_id]
+        if to_release_contents:
+            to_release_status = {}
+            for st in status_to_check:
+                if st.name not in to_release_status:
+                    to_release_status[st.name] = []
+
+            for to_release_content in to_release_contents:
+                for st in status_to_check:
+                    if (to_release_content['status'] in [st] or to_release_content['substatus'] in [st]):
+                        to_release_status[st.name].append(to_release_content['name'])
+
+            print("to_release_status: %s" % str(to_release_status))
+
+            contents = orm_contents.get_input_contents(request_id=to_release_contents[0]['request_id'],
+                                                       coll_id=to_release_contents[0]['coll_id'],
+                                                       name=None)
+            # print("contents: %s" % str(contents))
+
+            for content in contents:
+                if (content['content_relation_type'] == ContentRelationType.InputDependency):    # noqa: W503
+                    for st in status_to_check:
+                        if (content['status'] not in [st] and content['name'] in to_release_status[st.name]):
+                            update_content = {'content_id': content['content_id'],
+                                              'substatus': st,
+                                              'status': st}
+                            update_contents.append(update_content)
+
     return update_contents
 
 
