@@ -1,5 +1,5 @@
 
-import os
+import os   # noqa E402
 import sys
 import datetime
 
@@ -8,20 +8,34 @@ os.environ['PANDA_URL_SSL'] = 'https://ai-idds-01.cern.ch:25443/server/panda'
 
 from pandatools import Client  # noqa E402
 
+jobids = [1408118]
+jobs_list_status = Client.getJobStatus(jobids, verbose=0)
+print(jobs_list_status)
+jobs_list = jobs_list_status[1]
+print(jobs_list)
+for job_info in jobs_list:
+    if job_info is not None:
+        # if job_info.Files and len(job_info.Files) > 0:
+        print(job_info)
+        print(job_info.attemptNr)
+        print(job_info.maxAttempt)
+        print(job_info.Files)
+        print(job_info.Files[0])
+        for f in job_info.Files:
+            # print(dir(f))
+            print(f._attributes)
+            print(f.values())
+            print(f.type)
+
+
+jediTaskID = 3885
+ret = Client.getJediTaskDetails({'jediTaskID': jediTaskID}, True, True, verbose=False)
+print(ret)
+
+ret = Client.getTaskStatus(jediTaskID, verbose=False)
+print(ret)
 
 """
-jobids = [1276493]
-jobs_list = Client.getJobStatus(jobids, verbose=0)[1]
-for job_info in jobs_list:
-    # if job_info.Files and len(job_info.Files) > 0:
-    print(job_info)
-    print(job_info.Files)
-    print(job_info.Files[0])
-    for f in job_info.Files:
-        # print(dir(f))
-        print(f._attributes)
-        print(f.values())
-        print(f.type)
 sys.exit(0)
 
 jediTaskID = 998
@@ -40,6 +54,7 @@ ret = Client.getJediTaskDetails({'jediTaskID': jediTaskID}, True, True, verbose=
 print(ret)
 """
 
+task_ids = []
 # task_ids = [1565, 1566, 1567, 1568, 1570, 1572, 1575, 1576, 1579, 1580, 1581, 1582, 1584, 1585, 1586, 1587, 1588, 1589, 1590, 1591, 1592, 1593, 1597, 1598, 1599, 1601, 1602, 1603, 1604, 1607, 1608, 1609, 1610, 1611, 1612, 1613, 1617]
 # task_ids = [i for i in range(1091, 1104)] + [i for i in range(1274, 1392)]
 # task_ids = [812]
@@ -48,9 +63,12 @@ print(ret)
 # task_ids += [i for i in range(833, 839)]
 # task_ids += [i for i in range(1048, 1078)]
 # task_ids = [i for i in range(1855, 1856)]
-task_ids = [i for i in range(1840, 1850)] + [i for i in range(1990, 2000)]
+# task_ids = [i for i in range(1840, 1850)] + [i for i in range(1990, 2000)]
 # task_ids = [2549, 2560]
+# task_ids = [i for i in range(3692, 3723)]
+task_ids = []
 for task_id in task_ids:
+    print("Killing %s" % task_id)
     Client.killTask(task_id)
 
 """
@@ -67,8 +85,8 @@ Client.getFullJobStatus(ids, verbose=False)
 
 start_time = datetime.datetime.utcnow() - datetime.timedelta(hours=5)
 start_time = start_time.strftime('%Y-%m-%d %H:%M:%S')
-ret = Client.getJobIDsJediTasksInTimeRange(start_time, verbose=False)
-print(ret)
+# ret = Client.getJobIDsJediTasksInTimeRange(start_time, verbose=False)
+# print(ret)
 
 # ret = Client.getJobIDsJediTasksInTimeRange(start_time, task_type='test', verbose=False)
 # print(ret)
@@ -85,6 +103,7 @@ newOpts = {}
 # excludedSite = newOpts.get('excludedSite', None)
 # for JEDI
 """
+taskID=26034796
 status, out = Client.retryTask(taskID, verbose=True, properErrorCode=True, newParams=newOpts)
 print(status)
 print(out)
